####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(pROC)                 # for ROC curve for duplicity test
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data (gDistance)
# library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
# library(sampSurf)            # create spCircle for coverArea



duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt_lambda.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_RSS_uniform.dt_lambda.rds'))[
  , model := 'RSS'][
  , prior := 'uniform'][
  , deviceID := as.character(deviceID)]
duplicity1to1_RSS_network.dt_lambda.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_RSS_network.dt_lambda.rds'))[
  , model := 'RSS'][
  , prior := 'network'][
  , deviceID := as.character(deviceID)]
duplicity1to1_SDM_uniform.dt_lambda.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_SDM_uniform.dt_lambda.rds'))[
  , model := 'SDM'][
  , prior := 'uniform'][
  , deviceID := as.character(deviceID)]
duplicity1to1_SDM_network.dt_lambda.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_SDM_network.dt_lambda.rds'))[
  , model := 'SDM'][
  , prior := 'network'][
  , deviceID := as.character(deviceID)]

duplicity1to1_lambda.dt <- rbindlist(list(
  duplicity1to1_RSS_uniform.dt_lambda.dt,duplicity1to1_RSS_network.dt_lambda.dt,
  duplicity1to1_SDM_uniform.dt_lambda.dt,duplicity1to1_SDM_network.dt_lambda.dt))

duplicity1to1_lambda.dt <- merge(duplicity1to1_lambda.dt, duplicity1to1.dt[, .(deviceID, real_dup, model, prior)],
                                 by = c('deviceID', 'model', 'prior'), all.x = TRUE)[
  dupP > 0.5 & real_dup == "0", case := "FP"][
  dupP <= 0.5 & real_dup == "1", case := "FN"][
  dupP <= 0.5 & real_dup == "0", case := "TN"][
  dupP > 0.5 & real_dup == "1", case := "TP"]
                                 ]
duplicity1to1_cases.dt <- duplicity1to1_lambda.dt[, .N, by = c('model', 'prior', 'lambda','case')]


ggplot(duplicity1to1_cases.dt, aes(x = lambda, y = N, col = case)) +
  geom_point() +
  geom_vline(xintercept = 0.85 / 0.15, col = 'red') +
  facet_grid((model ~ prior)) +
  labs(x = TeX('$\\lambda$'), y = 'Number of cases') +
  theme_bw()
