####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postLocProb  <- file.path(path_root, 'postLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
 library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))

duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']
duplicity1to1_RSS_network.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'network']
duplicity1to1_SDM_uniform.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'uniform']
duplicity1to1_SDM_network.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'network']

duplicity1to1_FNcorrected.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_FNcorrected.dt.rds'))
duplicity1to1_RSS_uniform_FNcorrected.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']
duplicity1to1_RSS_network_FNcorrected.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'network']
duplicity1to1_SDM_uniform_FNcorrected.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'uniform']
duplicity1to1_SDM_network_FNcorrected.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'network']



positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

map <- readRDS(file.path(path_intermediate, 'map.rds'))

gridParam <- readRDS(file.path(path_intermediate, 'gridParam.rds'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS DETECTED BY MNO               ####
positionsConnections.dt <- positionsConnections.dt[
  , tile := as.integer(tile)]
setnames(positionsConnections.dt, 'deviceID', 'device')
positionsConnections.dt <- merge(positionsConnections.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
persons_region.dt <- positionsConnections.dt[
  !duplicated(positionsConnections.dt, by = c('time', 'personID'))][
  , .N, by = c('time', 'region')][
  , region := factor(region)]
persons_region.dt <- merge(data.table(expand.grid(time = seq(0, 890, by = 10), 
                                                  region = factor(1:10))),
                           persons_region.dt, all = TRUE)[
                             is.na(N), N := 0]
setnames(persons_region.dt, 'N', 'Nnet0')

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(persons_region.dt, aes(x = time, y = Nnet0)) +
  geom_point() +
  geom_line() +
  facet_wrap(factor(region) ~ ., labeller = labeller(region = region.labs)) +
  labs(title = '', x = 'time (s)', y = TeX('$N^{net,0}$')) +
  theme_bw() +
  theme(plot.title = element_text(hjust = 0.5, size = 22),
        axis.title.x = element_text(size = 18),
        axis.text.x = element_text(size = 10),
        axis.title.y = element_text(size = 18),
        axis.text.y = element_text(size = 8))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####          ESTIMATE NUMBER OF INDIVIDUALS DETECTED BY THE NETWORK        ####
### * RSS_uniform                                                           ####
prefix <- 'postLocProb_RSS_uniform'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_uniform.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_RSS_uniform.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_RSS_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_RSS_uniform.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'RSS'][
  , prior := 'uniform']


### * RSS_network                                                           ####
prefix <- 'postLocProb_RSS_network'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_network.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_RSS_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_RSS_network.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_RSS_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_RSS_network.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_RSS_network.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'RSS'][
    , prior := 'network']

### * SDM_uniform                                                           ####
prefix <- 'postLocProb_SDM_uniform'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_SDM_uniform.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_SDM_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_SDM_uniform.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_SDM_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_SDM_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_SDM_uniform.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'SDM'][
    , prior := 'uniform']


### * SDM_network                                                           ####
prefix <- 'postLocProb_SDM_network'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_SDM_network.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_SDM_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_SDM_network.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_SDM_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_SDM_network.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_SDM_network.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'SDM'][
    , prior := 'network']


Nnet.dt <- rbindlist(list(
  Nnet_RSS_uniform.dt, Nnet_RSS_network.dt,
  Nnet_SDM_uniform.dt, Nnet_SDM_network.dt))
saveRDS(Nnet.dt, file.path(path_intermediate, 'Nnet.dt.rds'))
rm(Nnet_RSS_network.dt, Nnet_RSS_uniform.dt, Nnet_SDM_network.dt, Nnet_SDM_uniform.dt)
gc()

### * RSS_uniform_FNcorrected                                               ####
prefix <- 'postLocProb_RSS_uniform'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_uniform_FNcorrected.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_RSS_uniform_FNcorrected.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_RSS_uniform_FNcorrected.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_RSS_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_RSS_uniform_FNcorrected.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'RSS'][
    , prior := 'uniform']


### * RSS_network_FNcorrected                                               ####
prefix <- 'postLocProb_RSS_network'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_network_FNcorrected.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_RSS_network_FNcorrected.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_RSS_network_FNcorrected.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_RSS_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_RSS_network.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_RSS_network_FNcorrected.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'RSS'][
    , prior := 'network']

### * SDM_uniform_FNcorrected                                               ####
prefix <- 'postLocProb_SDM_uniform'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_SDM_uniform_FNcorrected.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_SDM_uniform_FNcorrected.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_SDM_uniform_FNcorrected.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_SDM_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_SDM_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_SDM_uniform_FNcorrected.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'SDM'][
    , prior := 'uniform']


### * SDM_network_FNcorrected                                               ####
prefix <- 'postLocProb_SDM_network'

fwrite(gridParam, file.path(path_postLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postLocProb, 'grid.csv')

fwrite(duplicity1to1_SDM_network_FNcorrected.dt[, .(deviceID, dupP)], 
       file.path(path_postLocProb, 'duplicity_SDM_network_FNcorrected.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postLocProb, 'duplicity_SDM_network_FNcorrected.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postLocProb, 'rg_SDM_network.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postLocProb, 'rg_SDM_network.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
Nnet_SDM_network_FNcorrected.dt <- rNnetEvent(n, grFileName, dpFileName, rgFileName, path_postLocProb, prefix, times)[
  , model := 'SDM'][
    , prior := 'network']


Nnet_FNcorrected.dt <- rbindlist(list(
  Nnet_RSS_uniform_FNcorrected.dt, Nnet_RSS_network_FNcorrected.dt,
  Nnet_SDM_uniform_FNcorrected.dt, Nnet_SDM_network_FNcorrected.dt))
saveRDS(Nnet_FNcorrected.dt, file.path(path_intermediate, 'Nnet_FNcorrected.dt.rds'))




####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####           PLOT DISTRIBUTIONS OF NUMBERS OF INDIVIDUALS                 ####
### * RSS_uniform                                                           ####
setnames(Nnet_RSS_uniform.dt, 'N', 'Nnet')
Nnet_RSS_uniform_region.dt <- merge(
  Nnet_RSS_uniform.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_uniform.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_RSS_uniform_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_RSS_uniform_CI.dt <- Nnet_RSS_uniform.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'RSS'][
           , prior := 'uniform'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_uniform_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * RSS_network                                                           ####
setnames(Nnet_RSS_network.dt, 'N', 'Nnet')
Nnet_RSS_network_region.dt <- merge(
  Nnet_RSS_network.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_network.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_RSS_network_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_RSS_network_CI.dt <- Nnet_RSS_network.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'RSS'][
           , prior := 'network'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_network_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * SDM_uniform                                                           ####
setnames(Nnet_SDM_uniform.dt, 'N', 'Nnet')
Nnet_SDM_uniform_region.dt <- merge(
  Nnet_SDM_uniform.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_uniform.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_SDM_uniform_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_SDM_uniform_CI.dt <- Nnet_SDM_uniform.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'SDM'][
           , prior := 'uniform'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_uniform_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * SDM_network                                                           ####
setnames(Nnet_SDM_network.dt, 'N', 'Nnet')
Nnet_SDM_network_region.dt <- merge(
  Nnet_SDM_network.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_network.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_SDM_network_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_SDM_network_CI.dt <- Nnet_SDM_network.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'SDM'][
           , prior := 'network'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_network_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))


### * RSS_uniform_FNcorrected                                                           ####
setnames(Nnet_RSS_uniform_FNcorrected.dt, 'N', 'Nnet')
Nnet_RSS_uniform_FNcorrected_region.dt <- merge(
  Nnet_RSS_uniform_FNcorrected.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_uniform_FNcorrected.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_RSS_uniform_FNcorrected_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_RSS_uniform_FNcorrected_CI.dt <- Nnet_RSS_uniform_FNcorrected.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'RSS'][
           , prior := 'uniform'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_uniform_FNcorrected_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * RSS_network_FNcorrected                                                           ####
setnames(Nnet_RSS_network_FNcorrected.dt, 'N', 'Nnet')
Nnet_RSS_network_FNcorrected_region.dt <- merge(
  Nnet_RSS_network_FNcorrected.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_network_FNcorrected.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_RSS_network_FNcorrected_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_RSS_network_FNcorrected_CI.dt <- Nnet_RSS_network_FNcorrected.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'RSS'][
           , prior := 'network'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_RSS_network_FNcorrected_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * SDM_uniform_FNcorrected                                                           ####
setnames(Nnet_SDM_uniform_FNcorrected.dt, 'N', 'Nnet')
Nnet_SDM_uniform_FNcorrected_region.dt <- merge(
  Nnet_SDM_uniform_FNcorrected.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_uniform_FNcorrected.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_SDM_uniform_FNcorrected_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_SDM_uniform_FNcorrected_CI.dt <- Nnet_SDM_uniform_FNcorrected.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'SDM'][
           , prior := 'uniform'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_uniform_FNcorrected_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

### * SDM_network_FNcorrected                                                           ####
setnames(Nnet_SDM_network_FNcorrected.dt, 'N', 'Nnet')
Nnet_SDM_network_FNcorrected_region.dt <- merge(
  Nnet_SDM_network_FNcorrected.dt[, region := factor(region, levels = 1:10)],
  persons_region.dt, by = c('time', 'region'), all.x = TRUE)

sampleTimes <- seq(0, 890, by = 10)[seq(1, 90, by = 10)]
time.labs <- paste0(sampleTimes, 's')
names(time.labs) <- sampleTimes
region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_network_FNcorrected.dt[time %in% sampleTimes], aes(x = Nnet)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  facet_grid(region ~ time, scales = 'free', labeller = labeller(time = time.labs, region = region.labs)) +
  geom_vline(data = Nnet_SDM_network_FNcorrected_region.dt[time %in% sampleTimes], aes(xintercept = Nnet0, color = 'red'), size = 1)  +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, NA)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N^{net}$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

Nnet_SDM_network_FNcorrected_CI.dt <- Nnet_SDM_network_FNcorrected.dt[
  , {CI_HDI <- ci(Nnet, method = 'HDI', ci = 0.95); 
  CI_low_HDI <- CI_HDI$CI_low; CI_high_HDI <- CI_HDI$CI_high;
  CI_ETI <- ci(Nnet, method = 'ETI', ci = 0.95); 
  CI_low_ETI <- CI_ETI$CI_low; CI_high_ETI <- CI_ETI$CI_high;
  list(CI_low_HDI = CI_low_HDI, CI_high_HDI = CI_high_HDI,
       CI_low_ETI = CI_low_ETI, CI_high_ETI = CI_high_ETI)}, by = .(time, region)][
         , model := 'SDM'][
           , prior := 'network'][
             persons_region.dt, on = c('time', 'region')]

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(Nnet_SDM_network_FNcorrected_CI.dt) +
  geom_linerange(aes(x = time, ymin = CI_low_HDI, ymax = CI_high_HDI), size = 0.8) +
  geom_pointrange(aes(x = time, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.2) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
  facet_grid(region ~ ., labeller = labeller(region = region.labs), scales = 'free') +
  labs(y = TeX('$N^{net}$'), x= 'time (s)') +
  #scale_x_discrete(breaks = seq(0, 890, by = 10)[seq(1, 90, by = 10)], labels = seq(0, 890, by = 10)[seq(1, 90, by = 10)]) +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))
