####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postJointLocProb  <- file.path(path_root, 'postJointLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
 library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))

duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']
duplicity1to1_RSS_network.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'network']
duplicity1to1_SDM_uniform.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'uniform']
duplicity1to1_SDM_network.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'network']

duplicity1to1_FNcorrected.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1_FNcorrected.dt.rds'))
duplicity1to1_RSS_uniform_FNcorrected.dt <- duplicity1to1_FNcorrected.dt[model == 'RSS' & prior == 'uniform']
duplicity1to1_RSS_network_FNcorrected.dt <- duplicity1to1_FNcorrected.dt[model == 'RSS' & prior == 'network']
duplicity1to1_SDM_uniform_FNcorrected.dt <- duplicity1to1_FNcorrected.dt[model == 'SDM' & prior == 'uniform']
duplicity1to1_SDM_network_FNcorrected.dt <- duplicity1to1_FNcorrected.dt[model == 'SDM' & prior == 'network']

positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

map <- readRDS(file.path(path_intermediate, 'map.rds'))

gridParam <- readRDS(file.path(path_intermediate, 'gridParam.rds'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####         COMPUTE OD MATRIX OF INDIVIDUALS DETECTED BY MNO               ####
devIDs_Nnet <- sort(unique(persons_parsed.dt[!is.na(deviceID_1)]$deviceID_1))

positionsConnections.dt <- positionsConnections.dt[
  , tile := as.integer(tile)][
  deviceID %chin% devIDs_Nnet]
setnames(positionsConnections.dt, 'deviceID', 'device')
positionsConnections.dt <- merge(positionsConnections.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
od_to.dt <- positionsConnections.dt[
  , .(tile, time, device, region)][
    , time := time - 10L][
      , time_to := time + 10L][
        time >= 0]
setnames(od_to.dt, c('tile', 'region'), c('tile_to', 'region_to'))
od_from.dt <- positionsConnections.dt[
  , .(tile, time, device, region)][
    , time_from := time]
setnames(od_from.dt, c('tile', 'region'), c('tile_from', 'region_from'))
od_region.dt <- merge(od_from.dt, od_to.dt, by = c('device', 'time'))[
  , time := NULL][
    , .N, by = c('time_from', 'time_to', 'region_from', 'region_to')]
setnames(od_region.dt, 'N', 'Nnet0')
od_region.dt <- merge(
  data.table(expand.grid(time_from = seq(0, 880, by = 10),
                         time_to = seq(10, 890, by = 10),
                         region_from = 1:10, 
                         region_to = 1:10)),
  od_region.dt, all.y = TRUE)[
    is.na(Nnet0), Nnet0 := 0]
setcolorder(od_region.dt, c('time_from', 'time_to', 'region_from', 'region_to', 'Nnet0'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####      ESTIMATE OD MATRICES FOR INDIVIDUALS DETECTED BY THE NETWORK      ####
### * RSS_uniform                                                           ####
prefix <- 'postLocJointProbDevice_RSS_uniform'

fwrite(gridParam, file.path(path_postJointLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postJointLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_uniform.dt[, .(deviceID, dupP)], 
       file.path(path_postJointLocProb, 'duplicity_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postJointLocProb, 'duplicity_RSS_uniform.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postJointLocProb, 'rg_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postJointLocProb, 'rg_RSS_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
NnetOD_RSS_uniform.dt <- rNnetEventOD2(n, dpFileName, rgFileName, path_postJointLocProb, prefix)[
  , model := 'RSS'][
  , prior := 'uniform']
saveRDS(NnetOD_RSS_uniform.dt, file.path(path_intermediate, 'NnetOD_RSS_uniform.dt.rds'))


### * RSS_uniform_FNcorrected                                               ####
prefix <- 'postLocJointProbDevice_RSS_uniform'

fwrite(gridParam, file.path(path_postJointLocProb, 'grid.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
grFileName <- file.path(path_postJointLocProb, 'grid.csv')

fwrite(duplicity1to1_RSS_uniform_FNcorrected.dt[, .(deviceID, dupP)], 
       file.path(path_postJointLocProb, 'duplicity_RSS_uniform_FNcorrected.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
dpFileName <- file.path(path_postJointLocProb, 'duplicity_RSS_uniform_FNcorrected.csv')

fwrite(centroidCoord.dt[, .(tile, region)], 
       file.path(path_postJointLocProb, 'rg_RSS_uniform.csv'),
       row.names = FALSE, col.names = TRUE, sep = ',')
rgFileName <- file.path(path_postJointLocProb, 'rg_RSS_uniform.csv')
times <- sort(unique(positionsConnections.dt$time))
n <- 1e4
NnetOD_RSS_uniform_FNcorrected.dt <- rNnetEventOD2(n, dpFileName, rgFileName, path_postJointLocProb, prefix)[
  , model := 'RSS'][
    , prior := 'uniform']
saveRDS(NnetOD_RSS_uniform_FNcorrected.dt, file.path(path_intermediate, 'NnetOD_RSS_uniform_FNcorrected.dt.rds'))




####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####         COMPUTE CI FOR OD MATRIX OF INDIVIDUALS DETECTED BY MNO        ####
### * RSS_uniform                                                           ####
NnetOD_CI_RSS_uniform.dt <- NnetOD_RSS_uniform.dt[
  , ci(Nnet, method = 'HDI', ci = 0.95), by = .(time_from, time_to, region_from, region_to)]
NnetOD_CI_RSS_uniform.dt <- merge(NnetOD_CI_RSS_uniform.dt, od_region.dt, by = c('time_from', 'time_to', 'region_from', 'region_to'), all.x = TRUE)[
  is.na(Nnet0), Nnet0 := 0][
  , CI_covered := ifelse(Nnet0 >= CI_low & Nnet0 <= CI_high, TRUE, FALSE)][
  , dist_to_CI := ifelse(CI_covered == TRUE, 0, ifelse(Nnet0 > CI_high, Nnet0 - CI_high, CI_low - Nnet0))]

ggplot(NnetOD_CI_RSS_uniform.dt) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N^{net}$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')



### * RSS_uniform_FNcorrected                                               ####
NnetOD_CI_RSS_uniform_FNcorrected.dt <- NnetOD_RSS_uniform_FNcorrected.dt[
  , ci(Nnet, method = 'HDI', ci = 0.95), by = .(time_from, time_to, region_from, region_to)]
NnetOD_CI_RSS_uniform_FNcorrected.dt <- merge(
  NnetOD_CI_RSS_uniform_FNcorrected.dt, od_region.dt, 
  by = c('time_from', 'time_to', 'region_from', 'region_to'), all.x = TRUE)[
  is.na(Nnet0), Nnet0 := 0][
  , CI_covered := ifelse(Nnet0 >= CI_low & Nnet0 <= CI_high, TRUE, FALSE)][
  , dist_to_CI := ifelse(CI_covered == TRUE, 0, ifelse(Nnet0 > CI_high, Nnet0 - CI_high, CI_low - Nnet0))]

ggplot(NnetOD_CI_RSS_uniform_FNcorrected.dt) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = Nnet0, ymin = Nnet0, ymax = Nnet0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N^{net}$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')
