####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postLocProb  <- file.path(path_root, 'postLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

source(file.path(path_source, 'ci_bnbinom.R'))
source(file.path(path_source, 'ci_nbinom.R'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))

postLocProb_RSS_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))


duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']
duplicity1to1_RSS_network.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'network']
duplicity1to1_SDM_uniform.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'uniform']
duplicity1to1_SDM_network.dt <- duplicity1to1.dt[model == 'SDM' & prior == 'network']

positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

map <- readRDS(file.path(path_intermediate, 'map.rds'))

gridParam <- readRDS(file.path(path_intermediate, 'gridParam.rds'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

simulation_xml.list <- readRDS(file.path(path_intermediate, 'simulation.xml.rds'))



####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS DETECTED BY MNO               ####
positionsConnections.dt <- positionsConnections.dt[
  , tile := as.integer(tile)]
setnames(positionsConnections.dt, 'deviceID', 'device')
positionsConnections.dt <- merge(positionsConnections.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
persons_region.dt <- positionsConnections.dt[
  !duplicated(positionsConnections.dt, by = c('time', 'personID'))][
    , .N, by = c('time', 'region')][
      , region := factor(region)]
persons_region.dt <- merge(data.table(expand.grid(time = seq(0, 890, by = 10), 
                                                  region = factor(1:10))),
                           persons_region.dt, all = TRUE)[
                             is.na(N), N := 0]
setnames(persons_region.dt, 'N', 'Nnet0')

region.labs <- paste('region', 1:10)
names(region.labs) <- 1:10
ggplot(persons_region.dt, aes(x = time, y = Nnet0)) +
  geom_point() +
  geom_line() +
  facet_wrap(factor(region) ~ ., labeller = labeller(region = region.labs)) +
  labs(title = '', x = 'time (s)', y = TeX('$N^{net,0}$')) +
  theme_bw() +
  theme(plot.title = element_text(hjust = 0.5, size = 22),
        axis.title.x = element_text(size = 18),
        axis.text.x = element_text(size = 10),
        axis.title.y = element_text(size = 18),
        axis.text.y = element_text(size = 8))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS IN TARGET POPULATION          ####
N.dt <- merge(persons_parsed.dt, centroidCoord.dt, by = 'tile', all.x = TRUE)[
  , .N, by = c('time', 'region')][
  order(time, region)]
setnames(N.dt, 'N', 'N0')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS DETECTED BY THE NETWORK       ####
Nnet.dt <- merge(
  persons_parsed.dt[time == 0][!is.na(deviceID_1)],
  centroidCoord.dt[, .(tile, region)], by = 'tile', all.x = TRUE)[
    , .(deviceID_1, region)][
    , .N, by = 'region'][
  order(region)]
setnames(Nnet.dt, 'N', 'Nnet0')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####          COMPUTE TRUE NUMBER OF DEVICES BY REGION                      ####
Ndev1.dt <- merge(
  persons_parsed.dt[time == 0][!is.na(deviceID_1)],
  centroidCoord.dt[, .(tile, region)], by = 'tile', all.x = TRUE)[
  , .(deviceID_1, deviceID_2, region)]

Ndev2.dt <- copy(Ndev1.dt)[
  !is.na(deviceID_2), .N, by = 'region']
setnames(Ndev2.dt, 'N', 'N2')
Ndev1.dt <- copy(Ndev1.dt)[
  !is.na(deviceID_1), .N, by = 'region']
setnames(Ndev1.dt, 'N', 'N1')
Ndev.dt <- Ndev1.dt[
  Ndev2.dt, on = 'region'][
  , N := N1 + N2][
  order(region), .(region, N)]
setnames(Ndev.dt, 'N', 'Ndev')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  COMPUTE DEDUPLICATION FACTORS                         ####
postLocProb_RSS_uniform_region.dt <- postLocProb_RSS_uniform.dt[
  time == 0][
  , time := NULL]
postLocProb_RSS_uniform_region.dt <- merge(
  postLocProb_RSS_uniform_region.dt,
  centroidCoord.dt[, .(tile, region)], by = 'tile', all.x = TRUE)[
    , list(prob = sum(eventLoc)), by = c('device', 'region')]
setnames(duplicity1to1_RSS_uniform.dt, 'deviceID', 'device')
omega_RSS_uniform_region.dt <- postLocProb_RSS_uniform_region.dt[
  duplicity1to1_RSS_uniform.dt[, .(device, dupP)], on = 'device'][
    , list(omega1 = sum(prob * (1 - dupP)) / sum(prob), 
           omega2 = sum(prob * dupP) / sum(prob)), by = 'region'][
  Ndev.dt, on = 'region']

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                         COMPUTE REGION AREAS                           ####
regionAreas.dt <- centroidCoord.dt[
  , .(tile, region)][
  , tileArea_km2 := 0.250^2][
  , list(regionArea_km2 = sum(tileArea_km2)), by = 'region']


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####      ESTIMATE NUMBER OF INDIVIDUALS IN TARGET POPULATION               ####
#### * Compute parameters                                                   ####
cv_DensReg <- seq(from = 0, to = 0.20, by = 0.05)
cv_DensReg[cv_DensReg == 0] <- 0.01
relBias_DensReg <- relBias_Nnet <- seq(from = -.20, to = 0.20, by = 0.05) 
N_initial.dt <- data.table(expand.grid(
  region = 1:10, relBias_Nnet = relBias_Nnet, relBias_DensReg = relBias_DensReg, cv_DensReg = cv_DensReg))[
    N.dt[time == 0][, time := NULL], on = 'region'][
    Nnet.dt, on = 'region'][
    , Nreg := N0 + round(relBias_DensReg * N0)][
    , Nnet := Nnet0 + round(relBias_Nnet * Nnet0)][
    omega_RSS_uniform_region.dt, on = 'region'][
    regionAreas.dt, on = 'region'][
    , pntRate := Ndev / Nreg][
    , dedupPntRate := (omega1 + 0.5 * omega2) * pntRate][
    order(region)][
    , alpha := dedupPntRate * Nreg][
    , beta := (1 - dedupPntRate) * Nreg][
    , factor := (sqrt(1 + (2 * cv_DensReg / (1 + relBias_DensReg))^2) - 1)][  
    , theta := 0.5 * (Nreg / regionArea_km2) * (1 + relBias_DensReg) * factor][
    , zeta := 2 / factor][
    , factor := NULL][
    , Q:= regionArea_km2 * theta / (1 + regionArea_km2 * theta)][
    , cv_DensReg := as.factor(cv_DensReg)][
    , relBias_DensReg := as.factor(relBias_DensReg)][
    , region := as.factor(region)]

#### * Credible intervals - Model obs_bnb                                   #### 
N_initial_CI_bnb.dt <- copy(N_initial.dt)[, row := .I][
  , c('ci_low_HDI', 'ci_high_HDI') := as.list(ci_bnbinom(Nnet, alpha, beta, ci = 0.95, method = 'HDI')), by = 'row'][
  , model := 'obs_betaNegBin']

#### * Credible intervals - Model obs_nb                                    #### 
N_initial_CI_nb.dt <- copy(N_initial.dt)[, row := .I][
  , c('ci_low_HDI', 'ci_high_HDI') := as.list(ci_nbinom(Nnet, alpha, beta, ci = 0.95, method = 'HDI')), by = 'row'][
    , model := 'obs_negBin']

#### * Credible intervals - Model state_nb                                  #### 
N_initial_CI_stateNb.dt <- copy(N_initial.dt)[, row := .I][
  , c('ci_low_HDI', 'ci_high_HDI') := as.list(ci_nbinom(Nnet, alpha, beta, Q, zeta, state_inc = TRUE, ci = 0.95, method = 'HDI')), by = 'row'][
    , model := 'state_negBin']

N_initial_CI.dt <- rbindlist(list(
  N_initial_CI_bnb.dt, N_initial_CI_nb.dt, N_initial_CI_stateNb.dt))
saveRDS(N_initial_CI.dt, file.path(path_intermediate, 'N_initial_CI.dt.rds'))

#### * Posterior distribution - Model obs_bnb                               ####
N_initial_hist_bnb.dt <- copy(N_initial.dt)[, row := .I][
  , list(N = Nnet + rbnbinom(1e4, Nnet + 1, alpha - 1, beta)), by = c('region', 'relBias_Nnet', 'relBias_DensReg', 'cv_DensReg')][
  , model := 'obs_betaNegBin'][
  N.dt[time == 0][, time := NULL][, region := factor(region)], on = 'region']


#### * Posterior distribution - Model obs_nb                                ####
N_initial_hist_nb.dt <- copy(N_initial.dt)[, row := .I][
  , list(N = Nnet + rnbinom(1e4, Nnet + 1, (alpha - 1) / (alpha + beta - 1))), by = c('region', 'relBias_Nnet', 'relBias_DensReg', 'cv_DensReg')][
  , model := 'obs_negBin'][
  N.dt[time == 0][, time := NULL][, region := factor(region)], on = 'region']

#### * Posterior distribution - Model state_nb                              ####
N_initial_hist_stateNb.dt <- copy(N_initial.dt)[, row := .I][
  , list(N = Nnet + rnbinom(1e4, Nnet + zeta + 1, 1 - beta * Q / (alpha + beta))), by = c('region', 'relBias_Nnet', 'relBias_DensReg', 'cv_DensReg')][
  , model := 'state_negBin'][
  N.dt[time == 0][, time := NULL][, region := factor(region)], on = 'region']

N_initial_hist.dt <- rbindlist(list(
  N_initial_hist_bnb.dt, N_initial_hist_nb.dt, N_initial_hist_stateNb.dt))
saveRDS(N_initial_hist.dt, file.path(path_intermediate, 'N_initial_hist.dt'))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####          PLOT NUMBER OF INDIVIDUALS IN TARGET POPULATION               ####
#### * Model obs_bnb                                                        ####
cv_DensReg <- seq(from = 0, to = 0.20, by = 0.05)
cv_DensReg[cv_DensReg == 0] <- 0.01
relBias_DensReg <- relBias_Nnet <- seq(from = -.20, to = 0.20, by = 0.05) 
biasNnet.labs <- paste('rb_Nnet =', relBias_Nnet)
names(biasNnet.labs) <- relBias_DensReg
biasNreg.labs <- paste('rr_Nreg =', relBias_DensReg)
names(biasNreg.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg

N_initial_plot_bnb_byCV.list <- lapply(cv_DensReg, function(cvNreg){

  p <- ggplot(N_initial_CI.dt[model == 'obs_betaNegBin' & cv_DensReg == cvNreg]) +
        geom_linerange(aes(x = region, ymin = ci_low_HDI, ymax = ci_high_HDI), size = 0.8) +
        geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
        scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
        facet_grid(relBias_DensReg ~ relBias_Nnet, 
                   labeller = labeller(relBias_DensReg = biasNreg.labs,
                                       relBias_Nnet = biasNnet.labs)) +
        labs(y = TeX('$N$'), x= 'region') +
        theme_bw() +
        theme(axis.text.x = element_text(size= 7, angle=0), 
              legend.title = element_blank(),
              legend.position = 'none',
              legend.text = element_text(size = 14),
              legend.direction = 'horizontal',
              legend.key = element_rect(size = 12))
  return(p)
})


#### * Model obs_nb                                                         ####
cv_DensReg <- seq(from = 0, to = 0.20, by = 0.05)
cv_DensReg[cv_DensReg == 0] <- 0.01
relBias_DensReg <- relBias_Nnet <- seq(from = -.20, to = 0.20, by = 0.05) 
biasNnet.labs <- paste('rb_Nnet =', relBias_Nnet)
names(biasNnet.labs) <- relBias_DensReg
biasNreg.labs <- paste('rr_Nreg =', relBias_DensReg)
names(biasNreg.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
N_initial_plot_nb_byCV.list <- lapply(cv_DensReg, function(cvNreg){
  
  p <- ggplot(N_initial_CI.dt[model == 'obs_negBin' & cv_DensReg == cvNreg]) +
    geom_linerange(aes(x = region, ymin = ci_low_HDI, ymax = ci_high_HDI), size = 0.8) +
    geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
    scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
    facet_grid(relBias_DensReg ~ relBias_Nnet, 
               labeller = labeller(relBias_DensReg = biasNreg.labs,
                                   relBias_Nnet = biasNnet.labs)) +
    labs(y = TeX('$N$'), x= 'region') +
    theme_bw() +
    theme(axis.text.x = element_text(size= 7, angle=0), 
          legend.title = element_blank(),
          legend.position = 'none',
          legend.text = element_text(size = 14),
          legend.direction = 'horizontal',
          legend.key = element_rect(size = 12))
  return(p)
})


#### * Model state_nb                                                       ####
cv_DensReg <- seq(from = 0, to = 0.20, by = 0.05)
cv_DensReg[cv_DensReg == 0] <- 0.01
relBias_DensReg <- relBias_Nnet <- seq(from = -.20, to = 0.20, by = 0.05) 
biasNnet.labs <- paste('rb_Nnet =', relBias_Nnet)
names(biasNnet.labs) <- relBias_DensReg
biasNreg.labs <- paste('rr_Nreg =', relBias_DensReg)
names(biasNreg.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
N_initial_plot_stateNb_byCV.list <- lapply(cv_DensReg, function(cvNreg){
  
  p <- ggplot(N_initial_CI.dt[model == 'state_negBin' & cv_DensReg == cvNreg]) +
    geom_linerange(aes(x = region, ymin = ci_low_HDI, ymax = ci_high_HDI), size = 0.8) +
    geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
    scale_y_continuous(expand = c(0, 0), limits = c(0, NA)) +
    facet_grid(relBias_DensReg ~ relBias_Nnet, 
               labeller = labeller(relBias_DensReg = biasNreg.labs,
                                   relBias_Nnet = biasNnet.labs)) +
    labs(y = TeX('$N$'), x= 'region') +
    theme_bw() +
    theme(axis.text.x = element_text(size= 7, angle=0), 
          legend.title = element_blank(),
          legend.position = 'none',
          legend.text = element_text(size = 14),
          legend.direction = 'horizontal',
          legend.key = element_rect(size = 12))
  return(p)
})
