####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postLocProb  <- file.path(path_root, 'postLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
 library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))


postLocProb_RSS_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))

duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']

positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

gridParam <- readRDS(file.path(path_intermediate, 'gridParam.rds'))

tileEquiv.dt <- readRDS(file.path(path_intermediate, 'tileEquiv.dt.rds'))

Nnet.dt <- readRDS(file.path(path_intermediate, 'Nnet.dt.rds'))[
  time == 0][
  , time := NULL][
  model == 'RSS' & prior == 'uniform']
setnames(Nnet.dt, 'N', 'Nnet')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS DETECTED BY MNO               ####
positionsConnections.dt <- positionsConnections.dt[
  , tile := as.integer(tile)]
setnames(positionsConnections.dt, 'deviceID', 'device')
positionsConnections.dt <- merge(positionsConnections.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
persons_region.dt <- positionsConnections.dt[
  !duplicated(positionsConnections.dt, by = c('time', 'personID'))][
  , .N, by = c('time', 'region')][
  , region := factor(region)]
persons_region.dt <- merge(
  data.table(expand.grid(time = seq(0, 890, by = 10), region = factor(1:10))),
  persons_region.dt, all = TRUE)[
  is.na(N), N := 0][
  time == 0][
  , time := NULL]
setnames(persons_region.dt, 'N', 'Nnet0')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS IN TARGET POPULATION          ####
N.dt <- merge(persons_parsed.dt, centroidCoord.dt, by = 'tile', all.x = TRUE)[
  , .N, by = c('time', 'region')][
  order(time, region)][
  time == 0][
  , time := NULL][
  , region := factor(region)]
setnames(N.dt, 'N', 'N0')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                    COMPUTE TRUE NUMBER OF DEVICES                      ####
Ndev1.dt <- merge(
  persons_parsed.dt[time == 0][!is.na(deviceID_1)],
  centroidCoord.dt[, .(tile, region)], by = 'tile', all.x = TRUE)[
    , .(deviceID_1, deviceID_2, region)]

Ndev2.dt <- copy(Ndev1.dt)[
  !is.na(deviceID_2), .N, by = 'region']
setnames(Ndev2.dt, 'N', 'N2')
Ndev1.dt <- copy(Ndev1.dt)[
  !is.na(deviceID_1), .N, by = 'region']
setnames(Ndev1.dt, 'N', 'N1')
Ndev.dt <- Ndev1.dt[
  Ndev2.dt, on = 'region'][
  , N := N1 + N2][
  order(region), .(region, N)]
setnames(Ndev.dt, 'N', 'Ndev')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                COMPUTE DEDUPLICATION FACTORS                           ####
postLocProb_RSS_uniform_region.dt <- postLocProb_RSS_uniform.dt[
  time == 0][
  , time := NULL]
postLocProb_RSS_uniform_region.dt <- merge(
  postLocProb_RSS_uniform_region.dt,
  centroidCoord.dt[, .(tile, region)], by = 'tile', all.x = TRUE)[
  , list(prob = sum(eventLoc)), by = c('device', 'region')]
setnames(duplicity1to1_RSS_uniform.dt, 'deviceID', 'device')
omega_RSS_uniform_region.dt <- postLocProb_RSS_uniform_region.dt[
  duplicity1to1_RSS_uniform.dt[, .(device, dupP)], on = 'device'][
  , list(omega1 = sum(prob * (1 - dupP)) / sum(prob), 
         omega2 = sum(prob * dupP) / sum(prob)), by = 'region'][
  Ndev.dt, on = 'region'][
  , region := as.factor(region)]

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                     COMPUTE REGION AREAS                               ####
regionAreas.dt <- centroidCoord.dt[
  , .(tile, region)][
  , tileArea_km2 := 0.250^2][
  , list(regionArea_km2 = sum(tileArea_km2)), by = 'region'][
  , region := as.factor(region)]


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                COMPUTE POSTERIOR DISTRIBUTION                          ####
#### * Parameters                                                           ####
cv_DensReg <- seq(from = 0, to = 0.20, by = 0.05)
cv_DensReg[cv_DensReg == 0] <- 0.01
relBias_DensReg <- seq(from = -.20, to = 0.20, by = 0.05) 
N_initial.dt <- data.table(expand.grid(
  region = factor(1:10), relBias_DensReg = relBias_DensReg, cv_DensReg = cv_DensReg))[
  N.dt, on = 'region']
N_initial.dt <- merge(N_initial.dt, Nnet.dt, by = 'region', all.y = TRUE, allow.cartesian = TRUE)[
  , Nreg := N0 + round(relBias_DensReg * N0)][
  omega_RSS_uniform_region.dt, on = 'region'][
  regionAreas.dt, on = 'region'][
  , pntRate := Ndev / Nreg][
  , dedupPntRate := (omega1 + 0.5 * omega2) * pntRate][
  order(region)][
  , alpha := dedupPntRate * Nreg][
  , beta := (1 - dedupPntRate) * Nreg][
  , factor := (sqrt(1 + (2 * cv_DensReg / (1 + relBias_DensReg))^2) - 1)][  
  , theta := 0.5 * (Nreg / regionArea_km2) * (1 + relBias_DensReg) * factor][
  , zeta := 2 / factor][
  , factor := NULL][
  , Q:= regionArea_km2 * theta / (1 + regionArea_km2 * theta)][
  , cv_DensReg := as.factor(cv_DensReg)][
  , relBias_DensReg := as.factor(relBias_DensReg)][
  , region := as.factor(region)]

#### * Modelo obs_bnb                                                       ####
N_hist_bnb.dt <- copy(N_initial.dt)[, row := .I][
  , list(region = region,
         relBias_DensReg = relBias_DensReg,
         cv_DensReg = cv_DensReg,
         Nnet = Nnet,
         N = Nnet + rbnbinom(1, Nnet + 1, alpha - 1, beta)), by = 'row'][
    , model := 'obs_betaNegBin']

#### * Modelo obs_nb                                                        ####
N_hist_nb.dt <- copy(N_initial.dt)[, row := .I][
  , list(region = region,
         relBias_DensReg = relBias_DensReg,
         cv_DensReg = cv_DensReg,
         Nnet = Nnet,
         N = Nnet + rnbinom(1, Nnet + 1, (alpha - 1) / (alpha + beta - 1))), by = 'row'][
           , model := 'obs_negBin']

#### * Modelo state_nb                                                      ####
N_hist_stateNb.dt <- copy(N_initial.dt)[, row := .I][
  , list(region = region,
         relBias_DensReg = relBias_DensReg,
         cv_DensReg = cv_DensReg,
         Nnet = Nnet,
         N = Nnet + rnbinom(1, Nnet + 1, (alpha - 1) / (alpha + beta - 1))), by = 'row'][
           , model := 'state_negBin']

N0_process_hist.dt <- rbindlist(list(N_hist_bnb.dt, N_hist_nb.dt, N_hist_stateNb.dt))
saveRDS(N0_process_hist.dt, file.path(path_intermediate, 'N0_process_hist.dt.rds'))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                    COMPUTE CREDIBLE INTERVALS                          ####
N_CI_bnb.dt <- N_hist_bnb.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('region', 'relBias_DensReg', 'cv_DensReg')]
N_CI_bnb.dt <- merge(N_CI_bnb.dt, N.dt, by = 'region', all.x = TRUE, allow.cartesian = TRUE)[
  , model := 'obs_betaNegBin']

N_CI_nb.dt <- N_hist_nb.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('region', 'relBias_DensReg', 'cv_DensReg')]
N_CI_nb.dt <- merge(N_CI_nb.dt, N.dt, by = 'region', all.x = TRUE, allow.cartesian = TRUE)[
  , model := 'obs_negBin']


N_CI_stateNb.dt <- N_hist_stateNb.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('region', 'relBias_DensReg', 'cv_DensReg')]
N_CI_stateNb.dt <- merge(N_CI_stateNb.dt, N.dt, by = 'region', all.x = TRUE, allow.cartesian = TRUE)[
  , model := 'state_negBin']

N0_process_CI.dt <- rbindlist(list(N_CI_bnb.dt, N_CI_nb.dt, N_CI_stateNb.dt))
saveRDS(N0_process_CI.dt, file.path(path_intermediate, 'N0_process_CI.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                       PLOT CREDIBLE INTERVALS                          ####
#### Model obs_bnb                                                          ####
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_CI_bnb.dt) +
  geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
  facet_grid(relBias_DensReg ~ cv_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 cv_DensReg = cv.labs),
             scales = 'free') +
  labs(y = TeX('$N$'), x= 'region') +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

#### Model obs_nb                                                           ####
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_CI_nb.dt) +
  geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
  facet_grid(relBias_DensReg ~ cv_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 cv_DensReg = cv.labs),
             scales = 'free') +
  labs(y = TeX('$N$'), x= 'region') +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

#### Model state_nb                                                         ####
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_CI_stateNb.dt) +
  geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
  facet_grid(relBias_DensReg ~ cv_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 cv_DensReg = cv.labs),
             scales = 'free') +
  labs(y = TeX('$N$'), x= 'region') +
  theme_bw() +
  theme(axis.text.x = element_text(size= 8, angle=0), 
        legend.title = element_blank(),
        legend.position = 'none',
        legend.text = element_text(size = 14),
        legend.direction = 'horizontal',
        legend.key = element_rect(size = 12))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  PLOT POSTERIOR DISTRIBUTIONS                          ####
#### Model obs_bnb                                                          ####
region.labs <- paste('region = ', 1:10)
names(region.labs) <- as.character(1:10)
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_hist_bnb.dt[cv_DensReg == 0.05], aes(x = N)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  geom_vline(data = N_CI_bnb.dt[cv_DensReg == 0.05],  aes(xintercept = N0, color = 'red'), size = 1)  +
  facet_grid(region ~ relBias_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 region = region.labs),
             scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, 150)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### Model obs_nb                                                           ####
region.labs <- paste('region = ', 1:10)
names(region.labs) <- as.character(1:10)
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_hist_nb.dt[cv_DensReg == 0.05], aes(x = N)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  geom_vline(data = N_CI_nb.dt[cv_DensReg == 0.05],  aes(xintercept = N0, color = 'red'), size = 1)  +
  facet_grid(region ~ relBias_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 region = region.labs),
             scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, 150)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### Model state_nb                                                         ####
region.labs <- paste('region = ', 1:10)
names(region.labs) <- as.character(1:10)
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
ggplot(N_hist_stateNb.dt[cv_DensReg == 0.05], aes(x = N)) +
  geom_histogram(aes(y=..density..), binwidth = 1, alpha = 0.4) +
  geom_vline(data = N_CI_stateNb.dt[cv_DensReg == 0.05],  aes(xintercept = N0, color = 'red'), size = 1)  +
  facet_grid(region ~ relBias_DensReg, 
             labeller = labeller(relBias_DensReg = bias.labs,
                                 region = region.labs),
             scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  scale_x_continuous(limits = c(0, 150)) +
  #  scale_color_manual(values = c('blue', 'red', 'green'), labels = c('mean','true', 'MAP'), name = '') +
  theme_bw() +
  labs(x = TeX('$N$'), y = '', title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_blank(),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')
