####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postLocProb  <- file.path(path_root, 'postLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))


postLocProb_RSS_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))

duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']

positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

NnetOD.dt <- readRDS(file.path(path_intermediate, 'NnetOD_RSS_uniform.dt.rds'))

N_initial.dt <- readRDS(file.path(path_intermediate, 'N0_process_hist.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS DETECTED BY MNO               ####
positionsConnections.dt <- positionsConnections.dt[
  , tile := as.integer(tile)]
setnames(positionsConnections.dt, 'deviceID', 'device')
positionsConnections.dt <- merge(positionsConnections.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
persons_region.dt <- positionsConnections.dt[
  !duplicated(positionsConnections.dt, by = c('time', 'personID'))][
    , .N, by = c('time', 'region')][
      , region := factor(region)]
Nnet0.dt <- merge(
  data.table(expand.grid(time = seq(0, 890, by = 10), region = factor(1:10))),
  persons_region.dt, all = TRUE)[
    is.na(N), N := 0]
setnames(Nnet0.dt, 'N', 'Nnet0')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE TRUE NUMBER OF INDIVIDUALS IN TARGET POPULATION          ####
N0.dt <- merge(persons_parsed.dt, centroidCoord.dt, by = 'tile', all.x = TRUE)[
  , .N, by = c('time', 'region')][
  order(time, region)][
  , region := factor(region)]
setnames(N0.dt, 'N', 'N0')


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                COMPUTE POSTERIOR DISTRIBUTION                          ####
NnetOD_totalFrom.dt <- NnetOD.dt[
  , list(Nnet_totalFrom = sum(Nnet)), by = c('time_from', 'time_to', 'region_from', 'iter')]
tauOD.dt <- merge(NnetOD.dt, NnetOD_totalFrom.dt, 
                   by = c('time_from', 'time_to', 'region_from', 'iter'))[
  , tau_Nnet := Nnet / Nnet_totalFrom][
  Nnet_totalFrom == 0, tau_Nnet := 0][
  , model := NULL][
  , prior := NULL][
  , region_from := factor(region_from)][
  , region_to := factor(region_to)][
  , iter := as.integer(iter)][
  , Nnet_totalFrom := NULL][
  , Nnet := NULL]


N_initial.dt[
  , iter := rep(1:10000, times = nrow(N_initial.dt) / 10000)][
  , row := NULL]

times <- sort(unique(NnetOD.dt$time_from))
timeIncrement <- unique(diff(times))

#### * Model obs_bnb                                                        ####
N_CI_bnb.list <- vector(mode = 'list', length = length(times))
names(N_CI_bnb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
    , Nnet := NULL][
      , time := 0][
        model == 'obs_betaNegBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
N_CI_bnb_0.dt <- runningN.dt[
  , ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- runningN.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
  N_CI_bnb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

N_CI_bnb.dt <- rbindlist(list(N_CI_bnb_0.dt, rbindlist(N_CI_bnb.list)))
N_CI_bnb.dt <- merge(N_CI_bnb.dt, N0.dt, by = c('time', 'region'), all.x = TRUE)
rm(N_CI_bnb.list, N_CI_bnb_0.dt)

#### * Model obs_nb                                                        ####
N_CI_nb.list <- vector(mode = 'list', length = length(times))
names(N_CI_nb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
  , Nnet := NULL][
  , time := 0][
  model == 'obs_negBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
N_CI_nb_0.dt <- runningN.dt[
  , ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- runningN.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
  N_CI_nb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

N_CI_nb.dt <- rbindlist(list(N_CI_nb_0.dt, rbindlist(N_CI_nb.list)))
N_CI_nb.dt <- merge(N_CI_nb.dt, N0.dt, by = c('time', 'region'), all.x = TRUE)
rm(N_CI_nb.list, N_CI_nb_0.dt)

#### * Model obs_stateNb                                                        ####
N_CI_stateNb.list <- vector(mode = 'list', length = length(times))
names(N_CI_stateNb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
  , Nnet := NULL][
  , time := 0][
  model == 'state_negBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
N_CI_stateNb_0.dt <- runningN.dt[
  , ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- runningN.dt[, ci(N, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region', 'time', 'model')]
  N_CI_stateNb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

N_CI_stateNb.dt <- rbindlist(list(N_CI_stateNb_0.dt, rbindlist(N_CI_stateNb.list)))
N_CI_stateNb.dt <- merge(N_CI_stateNb.dt, N0.dt, by = c('time', 'region'), all.x = TRUE)
rm(N_CI_stateNb.list, N_CI_stateNb_0.dt)


N_CI.dt <- rbindlist(list(N_CI_bnb.dt, N_CI_nb.dt, N_CI_stateNb.dt))
saveRDS(N_CI.dt, file.path(path_intermediate, 'N_CI.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                       PLOT CREDIBLE INTERVALS                          ####
#### Model obs_bnb                                                          ####
N_CI_bnb.dt <- N_CI.dt[model == 'obs_betaNegBin']
relBias_DensReg <- unique(N_CI_bnb.dt$relBias_DensReg)
cv_DensReg <- unique(N_CI_bnb.dt$cv_DensReg)
bias.labs <- paste('rb_Nreg =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv_Nreg =', cv_DensReg)
names(cv.labs) <- cv_DensReg
N_max <- ceiling(unique(max(N_CI_bnb.dt$CI_high)) / 10) * 10
for(timeidx in seq(along = times)){
  timeInst <- times[timeidx]
  cat(paste0('Time:', timeInst, '... '))
  p <- ggplot(N_CI_bnb.dt[time == timeInst]) +
          geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
          geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
          facet_grid(relBias_DensReg ~ cv_DensReg, 
                     labeller = labeller(relBias_DensReg = bias.labs,
                                         cv_DensReg = cv.labs)) +
          scale_y_continuous(limits = c(0, N_max)) +
          labs(y = TeX('$N$'), x= 'region') +
          theme_bw() +
          theme(axis.text.x = element_text(size= 8, angle=0), 
                legend.title = element_blank(),
                legend.position = 'none',
                legend.text = element_text(size = 14),
                legend.direction = 'horizontal',
                legend.key = element_rect(size = 12))
   ggsave(plot = p, filename = file.path(path_image, paste0('N_CI_bnb_t-', timeidx, '.png')), width = 16, height = 9, dpi = 100)
   cat(' ok.\n')
}


#### Model obs_nb                                                           ####
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
N_max <- ceiling(unique(max(N_CI_nb.dt$CI_high)) / 10) * 10
for(timeidx in seq(along = times)){
  timeInst <- times[timeidx]
  cat(paste0('Time:', timeInst, '... '))
  p <- ggplot(N_CI_nb.dt[time == timeInst]) +
          geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
          geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
          facet_grid(relBias_DensReg ~ cv_DensReg, 
                     labeller = labeller(relBias_DensReg = bias.labs,
                                         cv_DensReg = cv.labs)) +
          scale_y_continuous(limits = c(0, N_max)) +
          labs(y = TeX('$N$'), x= 'region') +
          theme_bw() +
          theme(axis.text.x = element_text(size= 8, angle=0), 
                legend.title = element_blank(),
                legend.position = 'none',
                legend.text = element_text(size = 14),
                legend.direction = 'horizontal',
                legend.key = element_rect(size = 12))
  ggsave(plot = p, filename = file.path(path_image, paste0('N_CI_nb_t-', timeidx, '.png')), width = 16, height = 9, dpi = 100)
  cat(' ok.\n')
}
#### Model state_nb                                                         ####
bias.labs <- paste('relBias =', relBias_DensReg)
names(bias.labs) <- relBias_DensReg
cv.labs <- paste('cv =', cv_DensReg)
names(cv.labs) <- cv_DensReg
N_max <- ceiling(unique(max(N_CI_stateNb.dt$CI_high)) / 10) * 10
for(timeidx in seq(along = times)){
  timeInst <- times[timeidx]
  cat(paste0('Time:', timeInst, '... '))
  p <- ggplot(N_CI_stateNb.dt[time == timeInst]) +
          geom_linerange(aes(x = region, ymin = CI_low, ymax = CI_high), size = 0.8) +
          geom_pointrange(aes(x = region, y = N0, ymin = N0, ymax = N0, col = 'True Value'), size = 0.2) +
          facet_grid(relBias_DensReg ~ cv_DensReg, 
                     labeller = labeller(relBias_DensReg = bias.labs,
                                         cv_DensReg = cv.labs)) +
          labs(y = TeX('$N$'), x= 'region') +
          scale_y_continuous(limits = c(0, N_max)) +
          theme_bw() +
          theme(axis.text.x = element_text(size= 8, angle=0), 
                legend.title = element_blank(),
                legend.position = 'none',
                legend.text = element_text(size = 14),
                legend.direction = 'horizontal',
                legend.key = element_rect(size = 12))
  ggsave(plot = p, filename = file.path(path_image, paste0('N_CI_stateNb_t-', timeidx, '.png')), width = 16, height = 9, dpi = 100)
  cat(' ok.\n')
}

