####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                           SET PATHS                                    ####
path_root         <- 'N:/UDMTD/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
#path_root         <- 'N:/UDTMDCOM/Cursos.Seminarios/BIGSURV20/edj-paper/supplMaterial'
path_source       <- file.path(path_root, 'source')
path_input        <- file.path(path_root, 'simulatorInputData')
path_output       <- file.path(path_root, 'simulatorOutputData')
path_intermediate <- file.path(path_root, 'intermediateData')
path_image        <- file.path(path_root, 'image')
path_temp         <- file.path(path_root, 'temp')
path_postLocProb  <- file.path(path_root, 'postLocProb')

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                  LOAD PACKAGES AND FUNCTIONS                           ####
library(data.table)
library(destim)
library(stringr)
library(ggplot2)              # graphics
library(latex2exp)
library(pROC)                 # for ROC curve for duplicity test
library(extraDistr)
library(bayestestR)
# library(Matrix)
# library(ggimage)
# library(scales)
# library(raster)
# library(tidyr)                # transform xml documents to tables (tibbles)
# library(rgeos)                # spatial data
library(raster)               # spatial data
# library(sf)                   # spatial data
# library(gganimate)
# library(magick)
library(deduplication)
library(aggregation)

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####              READ INTERMEDIATE DATA (RDS FILES)                        ####

persons_parsed.dt <- readRDS(file.path(path_intermediate, "persons_parsed.dt.rds"))

events.dt <- readRDS(file.path(path_intermediate, 'events.dt.rds'))

centroidCoord.dt <- readRDS(file.path(path_intermediate, 'centroidCoord.dt.rds'))


postLocProb_RSS_uniform.dt <- readRDS(file.path(path_intermediate, 'postLocProb_RSS_uniform.dt.rds'))

duplicity1to1.dt <- readRDS(file.path(path_intermediate, 'duplicity1to1.dt.rds'))
duplicity1to1_RSS_uniform.dt <- duplicity1to1.dt[model == 'RSS' & prior == 'uniform']

positionsConnections.dt <- readRDS(file.path(path_intermediate, "positionsConnections.dt.rds"))

NnetOD.dt <- readRDS(file.path(path_intermediate, 'NnetOD_RSS_uniform.dt.rds'))

N_initial.dt <- readRDS(file.path(path_intermediate, 'N0_process_hist.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####       COMPUTE OD MATRIX OF INDIVIDUALS IN TARGET POPULATION            ####
persons_parsed.dt <- merge(persons_parsed.dt, centroidCoord.dt, by = c('tile', 'rasterCell'))
od_to.dt <- persons_parsed.dt[
  , .(tile, time, personID, region)][
    , time := time - 10L][
      , time_to := time + 10L][
        time >= 0]
setnames(od_to.dt, c('tile', 'region'), c('tile_to', 'region_to'))
od_from.dt <- persons_parsed.dt[
  , .(tile, time, personID, region)][
    , time_from := time]
setnames(od_from.dt, c('tile', 'region'), c('tile_from', 'region_from'))
od_region.dt <- merge(od_from.dt, od_to.dt, by = c('personID', 'time'))[
  , time := NULL][
    , .N, by = c('time_from', 'time_to', 'region_from', 'region_to')]
setnames(od_region.dt, 'N', 'OD0')
od_region.dt <- merge(
  data.table(expand.grid(time_from = seq(0, 880, by = 10),
                         time_to = seq(10, 890, by = 10),
                         region_from = 1:10, 
                         region_to = 1:10)),
  od_region.dt, all = TRUE)[
    is.na(OD0), OD0 := 0]
setcolorder(od_region.dt, c('time_from', 'time_to', 'region_from', 'region_to', 'OD0'))


####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                COMPUTE POSTERIOR DISTRIBUTION                          ####
NnetOD_totalFrom.dt <- NnetOD.dt[
  , list(Nnet_totalFrom = sum(Nnet)), by = c('time_from', 'time_to', 'region_from', 'iter')]
tauOD.dt <- merge(NnetOD.dt, NnetOD_totalFrom.dt, 
                  by = c('time_from', 'time_to', 'region_from', 'iter'))[
  , tau_Nnet := Nnet / Nnet_totalFrom][
  Nnet_totalFrom == 0, tau_Nnet := 0][
  , model := NULL][
  , prior := NULL][
  , region_from := factor(region_from)][
  , region_to := factor(region_to)][
  , iter := as.integer(iter)][
  , Nnet_totalFrom := NULL][
  , Nnet := NULL]


N_initial.dt[
  , iter := rep(1:10000, times = nrow(N_initial.dt) / 10000)][
  , row := NULL]

times <- sort(unique(NnetOD.dt$time_from))
timeIncrement <- unique(diff(times))

#### * Model obs_bnb                                                        ####
OD_CI_bnb.list <- vector(mode = 'list', length = length(times))
names(OD_CI_bnb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
  , Nnet := NULL][
  , time := 0][
  model == 'obs_betaNegBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  tempOD.dt <- tempDT[, OD := tau_Nnet * N]
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- tempOD.dt[
    , ci(OD, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region_from', 'region_to', 'time_from', 'time_to', 'model')]
  rm(tempOD.dt)
  OD_CI_bnb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

OD_CI_bnb.dt <- rbindlist(OD_CI_bnb.list)
OD_CI_bnb.dt <- merge(OD_CI_bnb.dt, od_region.dt[
  , region_from := factor(region_from)][
  , region_to := factor(region_to)], 
                      by = c('time_from', 'time_to', 'region_from', 'region_to'))
rm(OD_CI_bnb.list)


#### * Model obs_nb                                                        ####
OD_CI_nb.list <- vector(mode = 'list', length = length(times))
names(OD_CI_nb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
    , Nnet := NULL][
      , time := 0][
        model == 'obs_negBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  tempOD.dt <- tempDT[, OD := tau_Nnet * N]
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- tempOD.dt[
    , ci(OD, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region_from', 'region_to', 'time_from', 'time_to', 'model')]
  rm(tempOD.dt)
  OD_CI_nb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

OD_CI_nb.dt <- rbindlist(OD_CI_nb.list)
OD_CI_nb.dt <- merge(OD_CI_nb.dt, od_region.dt[
  , region_from := factor(region_from)][
    , region_to := factor(region_to)], 
  by = c('time_from', 'time_to', 'region_from', 'region_to'))
rm(OD_CI_nb.list)


#### * Model obs_stateNb                                                        ####
OD_CI_stateNb.list <- vector(mode = 'list', length = length(times))
names(OD_CI_stateNb.list) <- as.character(times)
runningN.dt <- copy(N_initial.dt)[
  , region := factor(region)][
    , Nnet := NULL][
      , time := 0][
        model == 'state_negBin']
setcolorder(runningN.dt, c('model', 'relBias_DensReg', 'cv_DensReg', 'region', 'iter', 'time'))
for (timeFrom in times){
  
  cat(paste0('Time ', timeFrom, '... '))  
  tempDT <- merge(tauOD.dt[time_from == timeFrom], runningN.dt,
                  by.x = c('region_from', 'iter'), by.y = c('region', 'iter'), 
                  allow.cartesian = TRUE)
  tempOD.dt <- tempDT[, OD := tau_Nnet * N]
  runningN.dt <- tempDT[, list(N = sum(tau_Nnet * N)), by = c('model', 'relBias_DensReg', 'cv_DensReg', 'region_to', 'iter')][
    , time := timeFrom + timeIncrement]
  setnames(runningN.dt, c('region_to'), c('region'))
  rm(tempDT)
  gc()
  tempCI.dt <- tempOD.dt[
    , ci(OD, ci = 0.95, method = 'HDI'), by = c('relBias_DensReg', 'cv_DensReg', 'region_from', 'region_to', 'time_from', 'time_to', 'model')]
  rm(tempOD.dt)
  OD_CI_stateNb.list[[as.character(timeFrom)]] <- tempCI.dt
  cat(paste0(' ok.\n'))
}

OD_CI_stateNb.dt <- rbindlist(OD_CI_stateNb.list)
OD_CI_stateNb.dt <- merge(OD_CI_stateNb.dt, od_region.dt[
  , region_from := factor(region_from)][
    , region_to := factor(region_to)], 
  by = c('time_from', 'time_to', 'region_from', 'region_to'))
rm(OD_CI_stateNb.list)

OD_CI.dt <- rbindlist(list(OD_CI_bnb.dt, OD_CI_nb.dt, OD_CI_stateNb.dt))
saveRDS(OD_CI.dt, file.path(path_intermediate, 'OD_CI.dt.rds'))

####  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: ####
####                      PLOT CREDIBLE INTERVALS                           ####
#### * model obs_bnb - cv=0.01 and rb=0                                     ####
ggplot(OD_CI_bnb.dt[cv_DensReg == 0.01 & relBias_DensReg == 0]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### * model obs_bnb - cv=0.2 and rb=0.2                                                    ####
ggplot(OD_CI_bnb.dt[cv_DensReg == 0.2 & relBias_DensReg == 0.2]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### * model obs_nb - cv=0.01 and rb=0                                     ####
ggplot(OD_CI_nb.dt[cv_DensReg == 0.01 & relBias_DensReg == 0]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### * model obs_nb - cv=0.2 and rb=0.2                                                    ####
ggplot(OD_CI_nb.dt[cv_DensReg == 0.2 & relBias_DensReg == 0.2]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### * model obs_stateNb - cv=0.01 and rb=0                                     ####
ggplot(OD_CI_stateNb.dt[cv_DensReg == 0.01 & relBias_DensReg == 0]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

#### * model obs_stateNb - cv=0.2 and rb=0.2                                                    ####
ggplot(OD_CI_stateNb.dt[cv_DensReg == 0.2 & relBias_DensReg == 0.2]) +
  geom_linerange(aes(x = time_from, ymin = CI_low, ymax = CI_high), size = 0.8) +
  geom_pointrange(aes(x = time_from, y = OD0, ymin = OD0, ymax = OD0, col = 'True Value'), size = 0.1) +
  facet_grid(region_from ~ region_to, scales = 'free') +
  scale_y_continuous(limits = c(0, NA)) +
  theme_bw() +
  labs(x = 'time (s)', y = TeX('$N$'), title = '') +
  theme(plot.title = element_text(hjust = 0.5, size = 18),
        axis.title.x = element_text(size = 16), axis.text.x = element_text(size = 9), 
        axis.title.y = element_text(size = 16), axis.text.y = element_text(size = 9),
        legend.title = element_blank(), legend.text = element_text(size = 12),
        legend.position = 'none')

